@@ -162,10 +162,10 @@ def get_img(fpath, ret_html=True, b64=True):
         return f'<img align="left" src="{fpath}">'
 
 def get_video(fpath, ret_html=True, b64=True):
+    mimetype = fpath.split(".")[-1]
     if b64:
-        mimetype = fpath.split(".")[-1]
-        with open(fpath, "rb") as videoFile:
-            fpath = f"data:video/{mimetype};base64," + base64.b64encode(videoFile.read()).decode()
+        with open(fpath, "rb") as f:
+            fpath = f"data:video/{mimetype};base64," + base64.b64encode(f.read()).decode()
 
     if not ret_html:
         return fpath
@@ -184,43 +184,65 @@ def convert_mediapath(fpath, b64=False):
         return fpath
 
 
+def get_parentname_fpaths(fpaths):
+    parentnames = [Path(fpath).parent.name for fpath in fpaths]
+    assert all(x == parentnames[0] for x in parentnames), f'inconsistent parent dir names: {parentnames}'
+    return parentnames[0]
+
+
 def main():
     parser = argparse.ArgumentParser(__doc__, formatter_class=argparse.ArgumentDefaultsHelpFormatter)
     parser.add_argument('data', type=str, nargs='+',
                         help='input table data. Format: columnname1 item1 item2 item3 , columname2 item1 item2 item3 ...')
     parser.add_argument('-b', '--base64', action='store_true')
-    parser.add_argument('-x', '--index', action='store_true', help='add an index column')
+    index_grp = parser.add_mutually_exclusive_group()
+    index_grp.add_argument('-x', '--index', action='store_true', help='add numerical index column')
+    index_grp.add_argument('-xf', '--filename_index', action='store_true', help='Infer index (rowname) based on row filenames, instead of numerical index. All columns must have identical filenames otherwise an error is raised')
     parser.add_argument('-a', '--infer_columns', action='store_true', help='(A)uto infer columnnames from parent directories')
+    parser.add_argument('-f', '--force', action='store_true', help='(F)orce or (F)ree mode: does not check for columns to have equal lengths')
+    parser.add_argument('--clamp', default=None, help='clamp number of columns choose a number or "min"')
     # parser.add_argument('--sort', action='store_true')
-    # parser.add_argument('-ts', '--trim_silence', action='store_true')
-
+    # parser.add_argument('-ts', '--trim_silence', action='store_true', help='trim silences in audios')
     args = parser.parse_args()
 
     # lol: list of lists, iterates columns wise
     lol_raw = [list(y) for x, y in itertools.groupby(args.data, lambda z: z == ',') if not x]
     
-    def get_parentname_fpaths(fpaths):
-        parentnames = [Path(fpath).parent.name for fpath in fpaths]
-        assert all(x == parentnames[0] for x in parentnames), f'inconsistent parent dir names: {parentnames}'
-        return parentnames[0]
-
     if args.infer_columns:
         lol = [[get_parentname_fpaths(l)] + l for l in lol_raw]
     else:
         lol = lol_raw
-
-    lol = [list(map(partial(convert_mediapath, b64=args.base64), l)) for l in lol]
-
     rowwise = np.array(lol).T
 
-    if args.index:
+    if args.filename_index:
+        rowwise = np.array([sorted(l) for l in rowwise.T]).T
+
+        rownames_raw = rowwise.T[:, 1:].tolist()
+        ## assert all columns have the same filenames
+        cols = [list(map(os.path.basename, l)) for l in rownames_raw]
+        for col in cols:
+            assert col == cols[0], f'Inconsistent cells when attempting --filename_index, expected:  \n{cols[0]}, got\n{col}'
+
+    if args.clamp is not None:
+        if args.clamp == 'min':
+            args.clamp = min(list(map(len, rowwise.T.tolist())))
+        print('args.clamp', args.clamp)
+        args.clamp = int(args.clamp)
+        rowwise = [l[:args.clamp-1] for l in rowwise]
+    
+    if not args.force:
+        for l in lol:
+            assert len(l) == len(lol[0]), f'inconsistent number of columns, expected {len(lol[0])}, got {len(l)}'
+
+    rowwise = np.array([list(map(partial(convert_mediapath, b64=args.base64), l)) for l in rowwise])
+
+    if args.filename_index:
+        index = np.array(['index'] + cols[0])[..., None]
+        rowwise = np.hstack((index, rowwise))
+    elif args.index:
         index = np.array(['index'] + list(range(len(rowwise) - 1)))[..., None]
         rowwise = np.hstack((index, rowwise))
 
-    for l in lol:
-        assert len(l) == len(lol[0]), f'inconsistent number of columns, expected {len(lol[0])}, got {len(l)}'
-
-    c, r = len(lol), len(lol[0])
 
     html = create_html(rowwise)
     real_print(html)
